==> idkit-0.2.4.tar.gz: lib/src/widget/gesture/widget_gesture_recognizer.dart <==
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:idkit/src/function/num/func_num.dart';

class IDKitGestureRecognizer extends StatefulWidget {
  const IDKitGestureRecognizer({
    super.key,
    this.child,
    this.onTap,
    this.onStart,
    this.onUpdate,
    this.onEnd,
    this.fingerCount,
    this.minPressure = 0,
    this.maxPressure = double.infinity,
    this.pressOnStart,
    this.pressOnUpdate,
    this.pressOnEnd,
    this.pressOnCancel,
    this.duration = const Duration(microseconds: 5000000),
  });

  /// Callback for click event.
  final Function(IDKitTapDetails details)? onTap;

  /// Gesture zoom, move start method call.
  final Function(IDKitStartDetails details)? onStart;

  /// Gesture zoom, mobile update method call.
  final Function(IDKitUpdateDetails details)? onUpdate;

  /// Gesture zooming and moving end method calls.
  final Function(IDKitEndDetails details)? onEnd;

  /// Callback for counting the number of fingers on the screen, one does not count.
  final Function(int)? fingerCount;

  /// Set the maximum and minimum range of pressure sensitivity
  final double minPressure;
  final double maxPressure;

  /// Screen pressure monitoring starts.
  final Function(PointerDownEvent event)? pressOnStart;

  /// On-Screen Pressure Monitoring Update.
  final Function(PointerMoveEvent event)? pressOnUpdate;

  /// Screen pressure monitoring ends.
  final Function(PointerUpEvent event)? pressOnEnd;

  /// End of pressure for touchscreen devices.
  final Function(PointerCancelEvent event)? pressOnCancel;

  /// finger racing time.
  final Duration duration;

  /// Sub widgets of the Gestures widget.
  final Widget? child;

  @override
  State<IDKitGestureRecognizer> createState() => _IDKitGestureRecognizerState();
}

class _IDKitGestureRecognizerState extends State<IDKitGestureRecognizer> {
  /// Record of fingers
  int serialTapDownCount = 0;
  int serialTapCancelCount = 0;

  /// Whether the operation is performed by more than 2 fingers
  bool isMultiFinger = false;

  /// Whether to perform zoom operation
  bool isZoom = false;

  /// Whether to start moving
  bool isMove = false;

  /// Pressure on touchscreen devices
  double pressure = 0;

  /// Record the initial point of the finger
  PrePoint initialPoint = const PrePoint();

  /// Types of touch screen devices.
  late PointerDeviceKind kind = PointerDeviceKind.unknown;

  /// Single Point Pressure Recording
  late double startPressure = 0;

  /// Record the initial set of points
  List<PrePoint> startPointList = [];

  @override
  Widget build(BuildContext context) {
    return Listener(
      onPointerDown: (PointerDownEvent event) {
        initialPoint = PrePoint(
          timeStamp: event.timeStamp,
          pointer: event.pointer,
          kind: event.kind,
          device: event.device,
          position: event.position,
          delta: event.delta,
          buttons: event.buttons,
          obscured: event.obscured,
          pressure: event.pressure.range(
            min: widget.minPressure,
            max: widget.maxPressure,
          ),
          pressureMin: event.pressureMin,
          pressureMax: event.pressureMax,
          distanceMax: event.distanceMax,
          distanceMin: event.distanceMin,
          size: event.size,
          radiusMajor: event.radiusMajor,
          radiusMinor: event.radiusMinor,
          radiusMin: event.radiusMin,
          radiusMax: event.radiusMax,
          orientation: event.orientation,
          tilt: event.tilt,
          platformData: event.platformData,
          synthesized: event.synthesized,
          embedderId: event.embedderId,
          localPosition: event.localPosition,
          localDelta: event.localDelta,
        );
        kind = event.kind;
        startPointList.clear();
        startPressure = event.pressure;
        widget.pressOnStart?.call(event);
      },
      onPointerMove: (PointerMoveEvent event) {
        kind = event.kind;
        if (isMove) {
          pressure = event.pressure;
        } else {
          if (event.pressure != startPressure) {
            final PrePoint startPoint = PrePoint(
              timeStamp: event.timeStamp,
              pointer: event.pointer,
              kind: event.kind,
              device: event.device,
              position: event.position,
              delta: event.delta,
              buttons: event.buttons,
              obscured: event.obscured,
              pressure: event.pressure.range(
                min: widget.minPressure,
                max: widget.maxPressure,
              ),
              pressureMin: event.pressureMin,
              pressureMax: event.pressureMax,
              distanceMax: event.distanceMax,
              distanceMin: event.distanceMin,
              size: event.size,
              radiusMajor: event.radiusMajor,
              radiusMinor: event.radiusMinor,
              radiusMin: event.radiusMin,
              radiusMax: event.radiusMax,
              orientation: event.orientation,
              tilt: event.tilt,
              platformData: event.platformData,
              synthesized: event.synthesized,
              embedderId: event.embedderId,
              localPosition: event.localPosition,
              localDelta: event.localDelta,
            );
            startPointList.add(startPoint);
          }
        }
        widget.pressOnUpdate?.call(event);
      },
      onPointerUp: (PointerUpEvent event) {
        kind = event.kind;
        pressure = event.pressure.range(
          min: widget.minPressure,
          max: widget.maxPressure,
        );
        widget.pressOnEnd?.call(event);
      },
      child: RawGestureDetector(
        gestures: {
          TapGestureRecognizer:
              GestureRecognizerFactoryWithHandlers<TapGestureRecognizer>(
            () => TapGestureRecognizer(),
            (instance) {
              instance.onTap = () {};
              instance.onTapDown = (TapDownDetails details) {};
              instance.onTapCancel = () {};
              instance.onTapUp = (TapUpDetails details) {};
            },
          ),
          ScaleGestureRecognizer:
              GestureRecognizerFactoryWithHandlers<ScaleGestureRecognizer>(
            () => ScaleGestureRecognizer(),
            (instance) {
              instance.onStart = (ScaleStartDetails details) {
                if (serialTapCancelCount == serialTapDownCount) {
                  serialTapCancelCount = serialTapDownCount = 0;
                }
                if (!isMultiFinger) isMultiFinger = details.pointerCount > 2;
                if (!isMultiFinger) {
                  if (isZoom) return;
                  isZoom = details.pointerCount == 2;
                  isMove = !isZoom;
                  if (startPointList.isEmpty) startPointList.add(initialPoint);
                  final IDKitStartDetails startDetails =
                      IDKitStartDetails.point(startPointList, kind, details);
                  widget.onStart?.call(startDetails);
                }
              };
              instance.onUpdate = (ScaleUpdateDetails details) {
                if (isZoom && details.pointerCount == 1) return;
                if (!isMultiFinger) {
                  final IDKitUpdateDetails updateDetails =
                      IDKitUpdateDetails.pressure(pressure, kind, details);
                  widget.onUpdate?.call(updateDetails);
                }
              };
              instance.onEnd = (ScaleEndDetails details) {
                if (details.pointerCount == 0) {
                  if (isMultiFinger) {
                    isMultiFinger = false;
                    return;
                  } else {
                    isZoom = isMove = false;
                  }
                  final IDKitEndDetails endDetails =
                      IDKitEndDetails.pressure(pressure, kind, details);
                  widget.onEnd?.call(endDetails);
                }
              };
            },
          ),
          SerialTapGestureRecognizer:
              GestureRecognizerFactoryWithHandlers<SerialTapGestureRecognizer>(
            () => SerialTapGestureRecognizer(),
            (SerialTapGestureRecognizer instance) {
              instance.onSerialTapDown = (details) => serialTapDownCount++;
              instance.onSerialTapUp = (SerialTapUpDetails details) {
                if (serialTapDownCount != serialTapCancelCount) {
                  if (serialTapDownCount == 1) {
                    widget.onTap
                        ?.call(IDKitTapDetails.pressure(pressure, details));
                  } else {
                    widget.fingerCount?.call(serialTapDownCount);
                  }
                }
                serialTapDownCount = serialTapCancelCount = 0;
              };
              instance.onSerialTapCancel =
                  (SerialTapCancelDetails details) => serialTapCancelCount++;
            },
          ),
        },
        child: widget.child,
      ),
    );
  }
}

/// Single point gesture
class IDKitTapDetails extends SerialTapUpDetails {
  IDKitTapDetails({
    this.pressure = 0,
    PointerDeviceKind? kind,
    Offset globalPosition = Offset.zero,
    Offset localPosition = Offset.zero,
    int count = 1,
  }) : super(
            globalPosition: globalPosition,
            localPosition: localPosition,
            kind: kind,
            count: count);

  /// Finger pressure on the screen.
  final double pressure;

  /// Easy to build
  factory IDKitTapDetails.pressure(
      double pressure, SerialTapUpDetails details) {
    return IDKitTapDetails(
      pressure: pressure,
      kind: details.kind,
      globalPosition: details.globalPosition,
      localPosition: details.localPosition,
      count: details.count,
    );
  }
}

/// Zoom and move start gesture information
class IDKitStartDetails extends ScaleStartDetails {
  IDKitStartDetails({
    this.points = const [],
    this.kind,
    Offset focalPoint = Offset.zero,
    Offset localFocalPoint = Offset.zero,
    int pointerCount = 0,
  }) : super(
            focalPoint: focalPoint,
            localFocalPoint: localFocalPoint,
            pointerCount: pointerCount);

  /// Finger pressure on the screen.
  final List<PrePoint> points;

  /// Types of touch screen devices.
  final PointerDeviceKind? kind;

  /// Zoom, Rotate, Move Gesture Details
  factory IDKitStartDetails.point(
    List<PrePoint> points,
    PointerDeviceKind? kind,
    ScaleStartDetails details,
  ) {
    return IDKitStartDetails(
      points: points,
      kind: kind,
      focalPoint: details.focalPoint,
      localFocalPoint: details.localFocalPoint,
      pointerCount: details.pointerCount,
    );
  }
}

/// Zoom and move update gesture information
class IDKitUpdateDetails extends ScaleUpdateDetails {
  IDKitUpdateDetails({
    this.pressure = 0,
    this.kind,
    Offset focalPoint = Offset.zero,
    Offset localFocalPoint = Offset.zero,
    Offset focalPointDelta = Offset.zero,
    double scale = 1.0,
    double horizontalScale = 1.0,
    double verticalScale = 1.0,
    double rotation = 0.0,
    int pointerCount = 0,
  }) : super(
          focalPoint: focalPoint,
          localFocalPoint: localFocalPoint,
          focalPointDelta: focalPointDelta,
          scale: scale,
          horizontalScale: horizontalScale,
          verticalScale: verticalScale,
          rotation: rotation,
          pointerCount: pointerCount,
        );

  /// Finger pressure on the screen.
  final double pressure;

  /// Types of touch screen devices.
  final PointerDeviceKind? kind;

  /// Easy to build
  factory IDKitUpdateDetails.pressure(
    double pressure,
    PointerDeviceKind? kind,
    ScaleUpdateDetails details,
  ) {
    return IDKitUpdateDetails(
      pressure: pressure,
      kind: kind,
      focalPoint: details.focalPoint,
      localFocalPoint: details.localFocalPoint,
      focalPointDelta: details.focalPointDelta,
      scale: details.scale,
      horizontalScale: details.horizontalScale,
      verticalScale: details.verticalScale,
      rotation: details.rotation,
      pointerCount: details.pointerCount,
    );
  }
}

/// Zoom and move end gesture information
class IDKitEndDetails extends ScaleEndDetails {
  IDKitEndDetails({
    this.pressure = 0,
    this.kind,
    Velocity velocity = Velocity.zero,
    int pointerCount = 0,
  }) : super(
          velocity: velocity,
          pointerCount: pointerCount,
        );

  /// Finger pressure on the screen.
  final double pressure;

  /// Types of touch screen devices.
  final PointerDeviceKind? kind;

  /// Easy to build
  factory IDKitEndDetails.pressure(
    double pressure,
    PointerDeviceKind? kind,
    ScaleEndDetails details,
  ) {
    return IDKitEndDetails(
      pressure: pressure,
      kind: kind,
      velocity: details.velocity,
      pointerCount: details.pointerCount,
    );
  }
}

class PrePoint {
  const PrePoint({
    this.timeStamp = Duration.zero,
    this.pointer = 0,
    this.kind = PointerDeviceKind.touch,
    this.device = 0,
    this.position = Offset.zero,
    this.delta = Offset.zero,
    this.buttons = kPrimaryButton,
    this.obscured = false,
    this.pressure = 0,
    this.pressureMin = 1,
    this.pressureMax = 1,
    this.distanceMax = 0,
    this.distanceMin = 0,
    this.size = 0,
    this.radiusMajor = 0,
    this.radiusMinor = 0,
    this.radiusMin = 0,
    this.radiusMax = 0,
    this.orientation = 0,
    this.tilt = 0,
    this.platformData = 0,
    this.synthesized = false,
    this.embedderId = 0,
    this.localPosition = Offset.zero,
    this.localDelta = Offset.zero,
  });

  final Duration timeStamp;
  final int pointer;
  final PointerDeviceKind kind;
  final int device;
  final Offset position;
  final Offset delta;
  final int buttons;
  final bool obscured;
  final double pressure;
  final double pressureMin;
  final double pressureMax;
  final double distanceMax;
  final double distanceMin;
  final double size;
  final double radiusMajor;
  final double radiusMinor;
  final double radiusMin;
  final double radiusMax;
  final double orientation;
  final double tilt;
  final int platformData;
  final bool synthesized;
  final int embedderId;
  final Offset localPosition;
  final Offset localDelta;
}

==> idkit-0.3.4.tar.gz: lib/src/widget/gesture/widget_gesture_recognizer.dart <==
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:idkit/idkit.dart';

class IDKitGestureRecognizer extends StatefulWidget {
  const IDKitGestureRecognizer({
    super.key,
    this.child,
    this.onTap,
    this.onStart,
    this.onUpdate,
    this.onEnd,
    this.fingerCount,
    this.minPressure = 0,
    this.maxPressure = double.infinity,
    this.pressOnStart,
    this.pressOnUpdate,
    this.pressOnEnd,
    this.pressOnCancel,
    this.duration = const Duration(microseconds: 5000000),
  });

  /// Callback for click event.
  final Function(IDKitTapDetails details)? onTap;

  /// Gesture zoom, move start method call.
  final Function(IDKitStartDetails details)? onStart;

  /// Gesture zoom, mobile update method call.
  final Function(IDKitUpdateDetails details)? onUpdate;

  /// Gesture zooming and moving end method calls.
  final Function(IDKitEndDetails details)? onEnd;

  /// Callback for counting the number of fingers on the screen, one does not count.
  final Function(int)? fingerCount;

  /// Set the maximum and minimum range of pressure sensitivity
  final double minPressure;
  final double maxPressure;

  /// Screen pressure monitoring starts.
  final Function(PointerDownEvent event)? pressOnStart;

  /// On-Screen Pressure Monitoring Update.
  final Function(PointerMoveEvent event)? pressOnUpdate;

  /// Screen pressure monitoring ends.
  final Function(PointerUpEvent event)? pressOnEnd;

  /// End of pressure for touchscreen devices.
  final Function(PointerCancelEvent event)? pressOnCancel;

  /// finger racing time.
  final Duration duration;

  /// Sub widgets of the Gestures widget.
  final Widget? child;

  @override
  State<IDKitGestureRecognizer> createState() => _IDKitGestureRecognizerState();
}

class _IDKitGestureRecognizerState extends State<IDKitGestureRecognizer> {
  /// Record of fingers
  int serialTapDownCount = 0;
  int serialTapCancelCount = 0;

  /// Whether the operation is performed by more than 2 fingers
  bool isMultiFinger = false;

  /// Whether to perform zoom operation
  bool isZoom = false;

  /// Whether to start moving
  bool isMove = false;

  /// Pressure on touchscreen devices
  double pressure = 0;

  /// Record the initial point of the finger
  PrePoint initialPoint = const PrePoint();

  /// Types of touch screen devices.
  late PointerDeviceKind kind = PointerDeviceKind.unknown;

  /// Single Point Pressure Recording
  late double startPressure = 0;

  /// Record the initial set of points
  List<PrePoint> startPointList = [];

  @override
  Widget build(BuildContext context) {
    return Listener(
      onPointerDown: (PointerDownEvent event) {
        initialPoint = PrePoint(
          timeStamp: event.timeStamp,
          pointer: event.pointer,
          kind: event.kind,
          device: event.device,
          position: event.position,
          delta: event.delta,
          buttons: event.buttons,
          obscured: event.obscured,
          pressure: event.pressure.range(
            min: widget.minPressure,
            max: widget.maxPressure,
          ),
          pressureMin: event.pressureMin,
          pressureMax: event.pressureMax,
          distanceMax: event.distanceMax,
          distanceMin: event.distanceMin,
          size: event.size,
          radiusMajor: event.radiusMajor,
          radiusMinor: event.radiusMinor,
          radiusMin: event.radiusMin,
          radiusMax: event.radiusMax,
          orientation: event.orientation,
          tilt: event.tilt,
          platformData: event.platformData,
          synthesized: event.synthesized,
          embedderId: event.embedderId,
          localPosition: event.localPosition,
          localDelta: event.localDelta,
        );
        kind = event.kind;
        startPointList.clear();
        startPressure = event.pressure;
        widget.pressOnStart?.call(event);
      },
      onPointerMove: (PointerMoveEvent event) {
        kind = event.kind;
        if (isMove) {
          pressure = event.pressure;
        } else {
          if (event.pressure != startPressure) {
            final PrePoint startPoint = PrePoint(
              timeStamp: event.timeStamp,
              pointer: event.pointer,
              kind: event.kind,
              device: event.device,
              position: event.position,
              delta: event.delta,
              buttons: event.buttons,
              obscured: event.obscured,
              pressure: event.pressure.range(
                min: widget.minPressure,
                max: widget.maxPressure,
              ),
              pressureMin: event.pressureMin,
              pressureMax: event.pressureMax,
              distanceMax: event.distanceMax,
              distanceMin: event.distanceMin,
              size: event.size,
              radiusMajor: event.radiusMajor,
              radiusMinor: event.radiusMinor,
              radiusMin: event.radiusMin,
              radiusMax: event.radiusMax,
              orientation: event.orientation,
              tilt: event.tilt,
              platformData: event.platformData,
              synthesized: event.synthesized,
              embedderId: event.embedderId,
              localPosition: event.localPosition,
              localDelta: event.localDelta,
            );
            startPointList.add(startPoint);
          }
        }
        widget.pressOnUpdate?.call(event);
      },
      onPointerUp: (PointerUpEvent event) {
        kind = event.kind;
        pressure = event.pressure.range(
          min: widget.minPressure,
          max: widget.maxPressure,
        );
        widget.pressOnEnd?.call(event);
      },
      child: RawGestureDetector(
        gestures: {
          TapGestureRecognizer:
              GestureRecognizerFactoryWithHandlers<TapGestureRecognizer>(
            () => TapGestureRecognizer(),
            (instance) {
              instance.onTap = () {};
              instance.onTapDown = (TapDownDetails details) {};
              instance.onTapCancel = () {};
              instance.onTapUp = (TapUpDetails details) {};
            },
          ),
          ScaleGestureRecognizer:
              GestureRecognizerFactoryWithHandlers<ScaleGestureRecognizer>(
            () => ScaleGestureRecognizer(),
            (instance) {
              instance.onStart = (ScaleStartDetails details) {
                if (serialTapCancelCount == serialTapDownCount) {
                  serialTapCancelCount = serialTapDownCount = 0;
                }
                if (!isMultiFinger) isMultiFinger = details.pointerCount > 2;
                if (!isMultiFinger) {
                  if (isZoom) return;
                  isZoom = details.pointerCount == 2;
                  isMove = !isZoom;
                  if (startPointList.isEmpty) startPointList.add(initialPoint);
                  final IDKitStartDetails startDetails =
                      IDKitStartDetails.point(startPointList, kind, details);
                  widget.onStart?.call(startDetails);
                }
              };
              instance.onUpdate = (ScaleUpdateDetails details) {
                if (isZoom && details.pointerCount == 1) return;
                if (!isMultiFinger) {
                  final IDKitUpdateDetails updateDetails =
                      IDKitUpdateDetails.pressure(pressure, kind, details);
                  widget.onUpdate?.call(updateDetails);
                }
              };
              instance.onEnd = (ScaleEndDetails details) {
                if (details.pointerCount == 0) {
                  if (isMultiFinger) {
                    isMultiFinger = false;
                    return;
                  } else {
                    isZoom = isMove = false;
                  }
                  final IDKitEndDetails endDetails =
                      IDKitEndDetails.pressure(pressure, kind, details);
                  widget.onEnd?.call(endDetails);
                }
              };
            },
          ),
          SerialTapGestureRecognizer:
              GestureRecognizerFactoryWithHandlers<SerialTapGestureRecognizer>(
            () => SerialTapGestureRecognizer(),
            (SerialTapGestureRecognizer instance) {
              instance.onSerialTapDown = (details) => serialTapDownCount++;
              instance.onSerialTapUp = (SerialTapUpDetails details) {
                if (serialTapDownCount != serialTapCancelCount) {
                  if (serialTapDownCount == 1) {
                    double tapPressure = 0;
                    if (startPointList.isNotEmpty) {
                      tapPressure = startPointList
                          .map((e) => e.pressure)
                          .toList()
                          .average;
                    }
                    widget.onTap
                        ?.call(IDKitTapDetails.pressure(tapPressure, details));
                  } else {
                    widget.fingerCount?.call(serialTapDownCount);
                  }
                }
                serialTapDownCount = serialTapCancelCount = 0;
              };
              instance.onSerialTapCancel =
                  (SerialTapCancelDetails details) => serialTapCancelCount++;
            },
          ),
        },
        child: widget.child,
      ),
    );
  }
}

/// Single point gesture
class IDKitTapDetails extends SerialTapUpDetails {
  IDKitTapDetails({
    this.pressure = 0,
    PointerDeviceKind? kind,
    Offset globalPosition = Offset.zero,
    Offset localPosition = Offset.zero,
    int count = 1,
  }) : super(
            globalPosition: globalPosition,
            localPosition: localPosition,
            kind: kind,
            count: count);

  /// Finger pressure on the screen.
  final double pressure;

  /// Easy to build
  factory IDKitTapDetails.pressure(
      double pressure, SerialTapUpDetails details) {
    return IDKitTapDetails(
      pressure: pressure,
      kind: details.kind,
      globalPosition: details.globalPosition,
      localPosition: details.localPosition,
      count: details.count,
    );
  }
}

/// Zoom and move start gesture information
class IDKitStartDetails extends ScaleStartDetails {
  IDKitStartDetails({
    this.points = const [],
    this.kind,
    Offset focalPoint = Offset.zero,
    Offset localFocalPoint = Offset.zero,
    int pointerCount = 0,
  }) : super(
            focalPoint: focalPoint,
            localFocalPoint: localFocalPoint,
            pointerCount: pointerCount);

  /// Finger pressure on the screen.
  final List<PrePoint> points;

  /// Types of touch screen devices.
  final PointerDeviceKind? kind;

  /// Zoom, Rotate, Move Gesture Details
  factory IDKitStartDetails.point(
    List<PrePoint> points,
    PointerDeviceKind? kind,
    ScaleStartDetails details,
  ) {
    return IDKitStartDetails(
      points: points,
      kind: kind,
      focalPoint: details.focalPoint,
      localFocalPoint: details.localFocalPoint,
      pointerCount: details.pointerCount,
    );
  }
}

/// Zoom and move update gesture information
class IDKitUpdateDetails extends ScaleUpdateDetails {
  IDKitUpdateDetails({
    this.pressure = 0,
    this.kind,
    Offset focalPoint = Offset.zero,
    Offset localFocalPoint = Offset.zero,
    Offset focalPointDelta = Offset.zero,
    double scale = 1.0,
    double horizontalScale = 1.0,
    double verticalScale = 1.0,
    double rotation = 0.0,
    int pointerCount = 0,
  }) : super(
          focalPoint: focalPoint,
          localFocalPoint: localFocalPoint,
          focalPointDelta: focalPointDelta,
          scale: scale,
          horizontalScale: horizontalScale,
          verticalScale: verticalScale,
          rotation: rotation,
          pointerCount: pointerCount,
        );

  /// Finger pressure on the screen.
  final double pressure;

  /// Types of touch screen devices.
  final PointerDeviceKind? kind;

  /// Easy to build
  factory IDKitUpdateDetails.pressure(
    double pressure,
    PointerDeviceKind? kind,
    ScaleUpdateDetails details,
  ) {
    return IDKitUpdateDetails(
      pressure: pressure,
      kind: kind,
      focalPoint: details.focalPoint,
      localFocalPoint: details.localFocalPoint,
      focalPointDelta: details.focalPointDelta,
      scale: details.scale,
      horizontalScale: details.horizontalScale,
      verticalScale: details.verticalScale,
      rotation: details.rotation,
      pointerCount: details.pointerCount,
    );
  }
}

/// Zoom and move end gesture information
class IDKitEndDetails extends ScaleEndDetails {
  IDKitEndDetails({
    this.pressure = 0,
    this.kind,
    Velocity velocity = Velocity.zero,
    int pointerCount = 0,
  }) : super(
          velocity: velocity,
          pointerCount: pointerCount,
        );

  /// Finger pressure on the screen.
  final double pressure;

  /// Types of touch screen devices.
  final PointerDeviceKind? kind;

  /// Easy to build
  factory IDKitEndDetails.pressure(
    double pressure,
    PointerDeviceKind? kind,
    ScaleEndDetails details,
  ) {
    return IDKitEndDetails(
      pressure: pressure,
      kind: kind,
      velocity: details.velocity,
      pointerCount: details.pointerCount,
    );
  }
}

class PrePoint {
  const PrePoint({
    this.timeStamp = Duration.zero,
    this.pointer = 0,
    this.kind = PointerDeviceKind.touch,
    this.device = 0,
    this.position = Offset.zero,
    this.delta = Offset.zero,
    this.buttons = kPrimaryButton,
    this.obscured = false,
    this.pressure = 0,
    this.pressureMin = 1,
    this.pressureMax = 1,
    this.distanceMax = 0,
    this.distanceMin = 0,
    this.size = 0,
    this.radiusMajor = 0,
    this.radiusMinor = 0,
    this.radiusMin = 0,
    this.radiusMax = 0,
    this.orientation = 0,
    this.tilt = 0,
    this.platformData = 0,
    this.synthesized = false,
    this.embedderId = 0,
    this.localPosition = Offset.zero,
    this.localDelta = Offset.zero,
  });

  final Duration timeStamp;
  final int pointer;
  final PointerDeviceKind kind;
  final int device;
  final Offset position;
  final Offset delta;
  final int buttons;
  final bool obscured;
  final double pressure;
  final double pressureMin;
  final double pressureMax;
  final double distanceMax;
  final double distanceMin;
  final double size;
  final double radiusMajor;
  final double radiusMinor;
  final double radiusMin;
  final double radiusMax;
  final double orientation;
  final double tilt;
  final int platformData;
  final bool synthesized;
  final int embedderId;
  final Offset localPosition;
  final Offset localDelta;
}
